#include "SearchForApp.h"

OSErr SearchVolForBrowser( short vRef,
							Boolean *gotItPtr,
							OSType **creators,
							short numCreators,
							FSSpecPtr specPtr,
							OSType *creatorPtr )
{
	short	i;
	DTPBRec	dPB;
	short	refNum;
	Str255	name;
	FInfo	fndrInfo;
	OSErr	err;
	char 	*cP;
	
	*gotItPtr = false;

	cP = (char*)&dPB;

	for ( i = 0; i < sizeof( dPB ); i++ ){
		*cP++ = 0;
	}
	
	dPB.ioNamePtr = (StringPtr)NULL;
	dPB.ioVRefNum = vRef;
	
	err = PBDTGetPath( &dPB );
	if ( err )
		return err;
	
	refNum = dPB.ioDTRefNum;
	
	dPB.ioCompletion = (IOCompletionUPP)NULL;
	dPB.ioNamePtr = name;
	
	for ( i = 0; i < numCreators; i++ ){
		dPB.ioFileCreator = (*creators)[ i ];
		
		dPB.ioIndex = 1;
		
		do{
			err = PBDTGetAPPL( &dPB, false );
			if ( err == afpItemNotFound || err == paramErr)
				break;
			if ( err )
				return err;
			
			// We got the application info, but it is possible that the application
			// has been deleted without the desktop file being updated.  Check to
			// see that the file is still there.
			
			err = HGetFInfo( vRef, dPB.ioAPPLParID, name, &fndrInfo );
			if ( !err ){
				err = FSMakeFSSpec( vRef, dPB.ioAPPLParID, name, specPtr );
				if ( err )
					return err;
				
				*creatorPtr = dPB.ioFileCreator;

				*gotItPtr = true;
				return noErr;
			}
		}while( !err );	
	}
	
	return noErr;
}

OSErr LaunchTheApp( FSSpecPtr specPtr )
{
	OSErr		err;
	Boolean		changed;
	AppleEvent		launchEvent;
	AppleEvent		replyEvent;
	AEDesc			aliasDesc;
	AEDesc			folderDesc;
	FSSpec			folderSpec;
	AEDesc			finderAddr;
	DescType		finderSig;
	AliasHandle		fAliasHdl;
	AliasHandle		appAliasHdl;
	AEDescList		aliasList;

	// Launch the browser by sending an Open Selection apple event to the Finder.
	// We need to do this so that the process shows up in the application menu, so that
	// we see zoomrects on launch, and so that the application's icon is dimmed in its
	// Finder window

	// Taken from Writeswell Jr.'s LaunchSpeller in DoChecking.c

	
	/* make an alias for the parent folder */
	
	err = FSMakeFSSpec( specPtr->vRefNum,
						specPtr->parID,
						(StringPtr)NULL,
						&folderSpec );
	if ( err ){
		return err;
	}
	
	err = NewAlias( (FSSpecPtr)NULL, &folderSpec, &fAliasHdl );
	if ( err ){
		return err;
	}

	/* Create the event to send to the Finder */
	
	finderSig = 'MACS';						/* Creator code of finder; type is 'FNDR' */
	err = AECreateDesc( typeApplSignature,
						(Ptr)&finderSig,
						sizeof( finderSig ),
						&finderAddr );
	if ( err ){
		return err;
	}	
				
	err = AECreateAppleEvent( kAEFinderEvents,
								kAEOpenSelection,
								&finderAddr,
								kAutoGenerateReturnID,
								kAnyTransactionID,
								&launchEvent );
	
	if ( err ){
		return err;
	}
	err = AEDisposeDesc( &finderAddr );
	if ( err ){
		return err;
	}
	
	/* Insert the folder alias record as the direct object of the batch event */

	/* First make a descriptor of the alias */
	HLock( (Handle) fAliasHdl );

	err = AECreateDesc( typeAlias,
						(Ptr)*fAliasHdl,
						(*fAliasHdl)->aliasSize,
						&folderDesc );

	HUnlock( (Handle) fAliasHdl );
	DisposHandle( (Handle) fAliasHdl );

	if ( err ){
		return err;
	}	
	
	err = AEPutParamDesc( &launchEvent,
							keyDirectObject,
							&folderDesc );
	if ( err ){
		return err;
	}
	err = AEDisposeDesc( &folderDesc );
	if ( err ){
		return err;
	}

	/* Put the application alias on the event */
	err = NewAlias( (FSSpecPtr)NULL, specPtr, &appAliasHdl );
	if ( err ){
		return err;
	}

	HLock( (Handle) appAliasHdl );

	err = AECreateDesc( typeAlias,
						(Ptr)*appAliasHdl,
						(*appAliasHdl)->aliasSize,
						&aliasDesc );

	HUnlock( (Handle) appAliasHdl );
	DisposeHandle( (Handle)appAliasHdl );

	if ( err ){
		return err;
	}	

	/* Now make a list of the alias descriptor.  There is only one element in this
	 * list
	 */
	
	err = AECreateList( (Ptr)NULL, (Size)0, false, &aliasList );
	
	if ( err ){
		return err;
	}
	
	err = AEPutDesc( &aliasList, 1L, &aliasDesc );
	
	if ( err ){
		return err;
	}

	err = AEDisposeDesc( &aliasDesc );
	if ( err ){
		return err;
	}
	
	err = AEPutParamDesc( &launchEvent,
							keySelection,
							&aliasList );
	if ( err ){
		return err;
	}

	err = AEDisposeDesc( &aliasList );
	if ( err ){
		return err;
	}

	err = AESend( &launchEvent,
					&replyEvent,
					kAEWaitReply + kAECanInteract,		/* Don't allow later switch */
					kAENormalPriority,
					1800,								// Thirty seconds; browser can take a long time to initialize
					(AEIdleUPP)NULL,
					(AEFilterUPP)NULL );
	
	if ( err ){
		return err;
	}
	err = AEDisposeDesc( &launchEvent );
	if ( err ){
		return err;
	}
	
	err = AEDisposeDesc( &replyEvent );
	if ( err ){
		return err;
	}
	
	return noErr;
}
